#!/bin/bash
#-------------------------------------------------------------------------
#                 Copyright 2018 Impro Technologies (Pty) Ltd
#                 P.O. Box 15407, Westmead, 3605
#-------------------------------------------------------------------------
# Project		:	Eureka
# File name		:	deploy.sh
# Author		:	Rashid Motala
#-------------------------------------------------------------------------
# Description  : This script is called by the Eureka Firmware Upgrade App
#                in order to properly deploy the upgrade artifacts onto
#                the board. It returns an exit value of 0 on success.
#                Last console output will be the text 'Upgrade Succeeded!'
#-------------------------------------------------------------------------

# Function to properly find the folder the script is running from 
get_script_dir () {
     SOURCE="${BASH_SOURCE[0]}"
     # While $SOURCE is a symlink, resolve it
     while [ -h "$SOURCE" ]; do
          DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"
          SOURCE="$( readlink "$SOURCE" )"
          # If $SOURCE was a relative symlink (so no "/" as prefix, need to
          # resolve it relative to the symlink base directory
          [[ $SOURCE != /* ]] && SOURCE="$DIR/$SOURCE"
     done
     DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"
     echo "$DIR" | tee /root/upgrade/upgrade.log
}

myname="$(basename "$0")"
echo "Executing Script" "$myname" | tee -a /root/upgrade/upgrade.log

# Redirect stderr to stdout
exec 2>&1

echo "Current progress is:5%" | tee -a /root/upgrade/upgrade.log

echo Script is running as "$(whoami)" | tee -a /root/upgrade/upgrade.log
script_dir="$(get_script_dir)"
if [ "$script_dir" != "/root/upgrade/upgrade_files" ]; then
  echo "Script is running from the incorrect folder"  "$script_dir"", upgrade failed!" | tee -a /root/upgrade/upgrade.log
  echo "The script must be run from the folder" "/root/upgrade/upgrade_files" | tee -a /root/upgrade/upgrade.log
  echo
  exit 1
fi

cd "$script_dir"

echo "Script is in folder" "$script_dir" | tee -a /root/upgrade/upgrade.log
echo "Current folder is" "$(pwd)" | tee -a /root/upgrade/upgrade.log
echo

ecode=0

# Check if upgrade is applicable to this version of the platform...
# If the major version of the upgrade package equals the major version of the platform,
# then allow the upgrade
product_issue=$(grep "product_issue" /etc/product_issue)
product_issue=${product_issue#"product_issue="}
major_issue=${product_issue%%.*}
major_issue="$(($major_issue))"
upgrade_version=$(grep "product_issue" product_issue)
upgrade_version=${upgrade_version#"product_issue="}
major_issue_upgrade=${upgrade_version%%.*}
major_issue_upgrade="$(($major_issue_upgrade))"
echo "Major Issue is" "$major_issue" "and Major Issue Upgrade is" "$major_issue_upgrade" | tee -a /root/upgrade/upgrade.log

if [ $major_issue -ne $major_issue_upgrade ]; then
    echo "Warning - Major version of upgrade does not match major version of product" | tee -a /root/upgrade/upgrade.log
    echo "Warning - This type of upgrade may be blocked in future" | tee -a /root/upgrade/upgrade.log
    # exit 1 # Carry on for now
fi

echo 

echo "Starting Upgrade..." | tee -a /root/upgrade/upgrade.log

echo "Current progress is:10%" | tee -a /root/upgrade/upgrade.log

date >> /root/package.history
cat "$script_dir"/"product_issue" >> /root/package.history

# Install all deb packages in deb_install_list.txt
while read filename ; do
    if [ -n "$filename" ]; then
        echo Installing package debs-upgrade/"$filename" | tee -a /root/upgrade/upgrade.log
        dpkg -i debs-upgrade/"$filename" >> /root/upgrade/upgrade.log | tee -a /root/upgrade/upgrade.log
        last_ecode="$?"
        echo "Exit status for installing" debs-upgrade/"$filename" is "$last_ecode"  | tee -a /root/upgrade/upgrade.log
        ecode=$(($ecode + $last_ecode))
    fi
done < "$script_dir"/deb_install_list.txt

echo "Exit status after installing deb packages is" "$ecode" | tee -a /root/upgrade/upgrade.log
echo

echo "Current progress is:30%" | tee -a /root/upgrade/upgrade.log

# Uncompress all tag.gz packages into filesystem root
for filename in targz-upgrade/*.tar.gz; do
    if [ ! -f "$filename" ]; then
        continue
    fi
    echo Uncompressing targz-upgrade/"$filename" | tee -a /root/upgrade/upgrade.log
    tar -C / -xvvf "$filename" | tee -a /root/upgrade/upgrade.log
    last_ecode="$?"
    echo "Exit status after unpacking" "$filename" is "$last_ecode" | tee -a /root/upgrade/upgrade.log
    ecode=$(($ecode + $last_ecode)) 
done

echo "Exit status after untarring is" "$ecode" | tee -a /root/upgrade/upgrade.log
echo

echo "Current progress is:50%" | tee -a /root/upgrade/upgrade.log

# Run all attached scripts
for filename in *.sh; do
    if [ "$myname" == "$filename" ]; then
        echo "Skipping self -" "$myname" | tee -a /root/upgrade/upgrade.log
        continue 
    fi
    echo Executing "$filename" | tee -a /root/upgrade/upgrade.log
    "$script_dir"/"$filename"
    last_ecode="$?"
    echo "Exit status after executing" "$filename" is "$last_ecode" | tee -a /root/upgrade/upgrade.log
    ecode=$(($ecode + $last_ecode)) 
done

echo "Exit status after running scripts is" "$ecode" | tee -a /root/upgrade/upgrade.log
echo

echo "Current progress is:70%" | tee -a /root/upgrade/upgrade.log

echo "Overall exit status is" "$ecode" | tee -a /root/upgrade/upgrade.log

if [ "$ecode" -ne 0 ]; then
    echo "Upgrade Failed!" | tee -a /root/upgrade/upgrade.log
    echo
    exit 1
fi

cp -v product_issue /etc/product_issue | tee -a /root/upgrade/upgrade.log
sync

echo "Current progress is:80%" | tee -a /root/upgrade/upgrade.log

echo "Upgrade Succeeded!" | tee -a /root/upgrade/upgrade.log
echo
./reboot_platform &

echo "Current progress is:90%" | tee -a /root/upgrade/upgrade.log

exit 0
